#include <msp/strings/utils.h>
#include "dir.h"
#include "utils.h"

using namespace std;

namespace Msp {
namespace FS {

string basename(const Path &p)
{
	return p[-1];
}

Path dirname(const Path &p)
{
	if(p.size()==1)
	{
		if(p.is_absolute())
			return p;
		return ".";
	}
	return p.subpath(0, p.size()-1);
}

string basepart(const string &fn)
{
	unsigned dot = fn.rfind('.');
	return fn.substr(0, dot);
}

string extpart(const string &fn)
{
	string::size_type dot = fn.rfind('.');
	if(dot==string::npos)
		return string();
	return fn.substr(dot);
}

Path fix_case(const Path &path)
{
	bool found = true;
	Path result;
	for(Path::Iterator i=path.begin(); i!=path.end(); ++i)
	{
		if(!found || *i=="/")
			result /= *i;
		else
		{
			list<string> files;
			if(result.size())
				files = list_files(result);
			else
				files = list_files(".");

			found = false;
			for(list<string>::iterator j=files.begin(); (j!=files.end() && !found); ++j)
				if(!strcasecmp(*j,*i))
				{
					result /= *j;
					found = true;
				}

			if(!found)
				result /= *i;
		}
	}

	return result;
}

Path relative(const Path &path, const Path &base)
{
	Path::Iterator i = path.begin();
	Path::Iterator j = base.begin();
	for(; (i!=path.end() && j!=base.end() && *i==*j); ++i, ++j) ;

	Path result;
	for(; j!=base.end(); ++j)
		result /= "..";
	for(; i!=path.end(); ++i)
		result /= *i;

	return result;
}

Path common_ancestor(const Path &path1, const Path &path2)
{
	Path::Iterator i = path1.begin();
	Path::Iterator j = path2.begin();
	Path result;
	for(; (i!=path1.end() && j!=path2.end() && *i==*j); ++i, ++j)
		result /= *i;
	return result;
}

int descendant_depth(const Path &path, const Path &parent)
{
	Path::Iterator i = path.begin();
	Path::Iterator j = parent.begin();
	for(; (i!=path.end() && j!=parent.end() && *i==*j); ++i, ++j) ;

	if(j!=parent.end())
		return -1;
	
	int result = 0;
	for(; i!=path.end(); ++i)
		++result;

	return result;
}

} // namespace FS
} // namespace Msp
