#include "console.h"

using namespace std;

namespace Msp {
namespace IO {

Console::Console(Stream s):
	stream(s)
{
	mode = (stream==CIN ? M_READ : M_WRITE);

	platform_init();

	if(stream==CIN)
		set_events(P_INPUT);
}

void Console::set_block(bool b)
{
	sys_set_blocking(handle, b);
}

unsigned Console::do_write(const char *buf, unsigned len)
{
	check_access(M_WRITE);

	return sys_write(handle, buf, len);
}

unsigned Console::do_read(char *buf, unsigned len)
{
	check_access(M_READ);

	unsigned ret = sys_read(handle, buf, len);
	if(ret==0)
		set_eof();

	return ret;
}

Console &Console::instance(Stream s)
{
	static Console in(CIN);
	static Console out(COUT);
	static Console err(CERR);

	switch(s)
	{
	case CIN: return in;
	case COUT: return out;
	case CERR: return err;
	}

	throw invalid_argument("Console::instance");
}

Console &cin = Console::instance(Console::CIN);
Console &cout = Console::instance(Console::COUT);
Console &cerr = Console::instance(Console::CERR);

} // namespace IO
} // namespace Msp
